   module read_parms
   implicit none
   integer, parameter :: dp=8
   real(dp), parameter :: uae=27.2113834d0
!!******* Parameters of Phys. Rev. B Vol 62 Number 8 page 5109      
      real(dp), parameter :: ESS000 =   -6.17334
      real(dp), parameter :: EPP000 =    2.39585
!!*************************************************
      real(dp), parameter :: ESS111 =   -1.78516
      real(dp), parameter :: ESX111 =    0.78088
      real(dp), parameter :: EXX111 =    0.35657
      real(dp), parameter :: EXY111 =    1.47649
!!*************************************************
      real(dp), parameter :: ESS220 =    0.23010
      real(dp), parameter :: ESX220 =   -0.21608
      real(dp), parameter :: ESX022 =   -0.02496
      real(dp), parameter :: EXX220 =    0.02286
      real(dp), parameter :: EXX022 =   -0.24379
      real(dp), parameter :: EXY220 =   -0.05462
      real(dp), parameter :: EXY022 =   -0.12754
!!*************************************************
      real(dp), parameter :: ESS311 =   -0.06857
      real(dp), parameter :: ESX311 =    0.25209
      real(dp), parameter :: ESX113 =   -0.17098
      real(dp), parameter :: EXX311 =    0.13968 
      real(dp), parameter :: EXX113 =   -0.04580
      real(dp), parameter :: EXY311 =   -0.03625 
      real(dp), parameter :: EXY113 =    0.06921
   end module read_parms


   program dielectric
   use read_parms
   implicit none
   
   integer model,ensteps,i,mu,nval,k,n1,n2,nk,norb,ncond,ndim,ik,b,n,c
   real(8) q(3),emin,emax,broad,estep,w,m_pi,acell,ual,vol,factor
   integer a1,a2
   integer first,last,d,l,iv,ic,ndime,j
   
   real(8), allocatable :: kpt(:,:),egv(:,:),os(:,:),egvBSE(:)


   real(8) kr,efield(3),x,y,z,deltae,ost,er
   
   complex(8), allocatable :: e(:),zvect(:,:,:),egvectBSE(:,:)
   real(8), allocatable :: e2(:)
   complex(8), allocatable :: A(:,:,:),OBSE(:,:,:)
   complex(8) temp 
   
   parameter(m_pi=3.14159265358979323846)
   
   parameter(ual=0.5291772083d0)

   open(unit=10,file='eigenvalues.dat',form='formatted',status='old') 
   open(unit=11,file='eigenvectors.dat',form='unformatted',status='old')
   open(unit=12,file='dielectric.input',form='formatted',status='old')
   open(unit=13,file='new_eigenvalues.dat',form='formatted',status='old') 
   open(unit=14,file='new_eigenvectors.dat',form='unformatted',status='old')

   read (10,'(/35x,i6)') nk
   read (10,'(35x,i6)') nval
   read (10,'(35x,i6)') ncond

   write(*,*) ' Number of k points = ',nk
   write(*,*) ' Number of valence  = ',nval
   write(*,*) ' Number of conductions = ',ncond
   
   ndim=nval+ncond

   norb=4
  
   ndime=nval*ncond*nk

   write(*,*) ' *** ALLOCATION MATRIX *** ',ndime
   
   ALLOCATE(kpt(3,nk),zvect(ndim,ndim,nk),egv(ndim,nk),A(8,8,nk))
   ALLOCATE(egvBSE(ndime),egvectBSE(ndime,ndime),OBSE(nval,ncond,nk))
   
   do ik=1,nk
    read(10,*) (kpt(i,ik),i=1,3),(egv(i,ik),i=1,ndim)
    read(11) ((zvect(c,b,ik),c=1,ndim),b=1,ndim)
   enddo

   close(10)
   close(11)
   
   do j=1,ndime
    read(13,*) egvBSE(j)
    read(14)  (egvectBSE(i,j),i=1,ndime)
   enddo

   close(13)
   close(14)
   
   write(*,*) ' Q vector : '
   read(12,*)
   read(12,*)   emin,emax   
   write(*,*)  'Energy range: ',emin,emax
   read(12,*)
   read(12,*)   broad   
   write(*,*)  'Broadening: ',broad
   read(12,*)
   read(12,*)   ensteps  
   write(*,*)  'Energy steps: ',ensteps 
   read(12,*)
   read(12,*)   first
   if(first.eq.0) first=1
   write(*,*)  'First Band: ',first 
   read(12,*)
   read(12,*)   last
   if(last.eq.0) last=nval+ncond
   write(*,*)  'Last Conduction Band: ',last
   read(12,*)
   read(12,*)   (efield(i),i=1,3)  
   
   if(first.gt.nval.or.last.gt.nval+ncond) then
           stop ' ERROR first or last band wrong! '
   endif
   
   allocate(e(ensteps),e2(ensteps))

   open(unit=13,file='eqw-bse.dat',form='formatted',status='unknown')

   estep=(emax-emin)/dble(ensteps)
  
   acell=5.341
   acell=acell/ual
   
   vol=1.d0/4.d0*acell**3
   factor=2.d0*(4.d0*m_pi)/vol
   nval=4
   
  
   call evalA(nk,kpt,efield,A,acell)

   call evalOBSE(nval,ncond,nk,OBSE,A,zvect)

  
   do n=1,ensteps
   w=emin+(n-1)*estep
   e(n)=DCMPLX(1.d0,0.d0)
   e2(n)=0.d0
  
   do l=1,ndime

      temp=(0.d0,0.d0)
      i=0
      do ik=1,nk
       do iv=1,nval
        do ic=1,ncond
        i=1+1
        temp=temp+OBSE(iv,ic,ik)*egvectBSE(i,l)
        enddo
       enddo
      enddo
         
        e(n)=e(n)+uae**3*DCMPLX(factor,0.d0) & !*temp*dconjg(temp) &
        &/(egvBSE(l)-w-DCMPLX(0.d0,broad)) 
       
!       write(*,*) ' egvBSE(l) ',egvBSE(l)
        e2(n)=e2(n)+uae**3*broad/m_pi/((w-egvBSE(l))**2+broad**2)&
       & *DCMPLX(factor,0.d0) !*temp*dconjg(temp)
       
     enddo
    enddo

   do i=1,ensteps
    w=emin+(i-1)*estep
    write(13,'(4e)') w,REAL(e(i)),DIMAG(e(i)),e2(i)
   enddo
   
   close(13)
   close(12)
   close(10)
   close(11)
   end program dielectric

   subroutine evalOBSE(nval,ncond,nk,OBSE,A,zvect,egv)
   implicit none
   integer, parameter :: dp=8
   integer, intent(in) :: nval,ncond,nk
   complex(dp) ,intent(in)  :: A(nval+ncond,nval+ncond,nk)
   complex(dp) ,intent(out) :: OBSE(nval,ncond,nk)
   complex(dp) ,intent(in)  :: zvect(nval+ncond,nval+ncond,nk)
   real(dp), intent(in) :: egv(nval+ncond,nk)
   integer a1,a2,ik,iv,ic
   complex(dp) temp
    
   do ik=1,nk
    do  iv=1,nval
     do  ic=nval+1,ncond
   
    temp=(0.d0,0.d0)
    do a1=1,nval+ncond
      do a2=1,nval+ncond
        temp=temp+dconjg(zvect(a1,ic,ik))*zvect(a2,iv,ik)*A(a1,a2,ik)
       enddo
      enddo
    temp=temp/DCMPLX(0.d0,1.d0)/(egv(iv,ik)-egv(ic,ik))**2

     OBSE(iv,ic,ik)=temp
     enddo
    enddo
   enddo
   end subroutine evalOBSE

   subroutine evalA(nk,kpt,efield,A,acell)
   implicit none
   integer, intent(in)  :: nk
   integer, parameter :: dp=8
   real(dp), intent(in)  :: kpt(3,nk),efield(3),acell
   real(dp), intent(out) :: A(8,8,nk)
   integer fnn(3,4),snn(3,12),tnn(3,12),difabs,ii,jj,k,i
   integer norb
   real(dp) kr,er
   real(dp), parameter :: uae=27.2113834d0
   complex(dp)  phase,HV(4,4)
   
   norb=4
   
   call findneighboars(fnn,snn,tnn)
    A=(0.d0,0.d0)
    do k=1,nk
!****** First Neighboard
        do i=1,4
           kr=kpt(1,k)*fnn(1,i)+kpt(2,k)*fnn(2,i)+kpt(3,k)*fnn(3,i) 
           kr=kr*acell/16.d0
          CALL INIH1(HV,FNN(1,I),FNN(2,I),FNN(3,I))                    
!************************************************************************
        
           er=(fnn(1,i)*efield(1)+fnn(2,i)*efield(2)+fnn(3,i)*efield(3))

        DO  II=1,norb                                                
          DO  JJ=1,norb                                              
      A(ii,jj+norb,k)= A(ii,jj+norb,k)+HV(II,JJ)*DCMPLX(dcos(kr),dsin(kr)) &
     &*er*acell/16.d0/uae
     
       A(ii+norb,jj,k)= A(ii+norb,jj,k)-HV(II,JJ)*DCMPLX(dcos(-kr),dsin(-kr)) &
     &*er*acell/16.d0/uae
          enddo
         enddo
!************************************************************************
        enddo
!******
!****** Second Neighboard
        do i=1,12
           kr=kpt(1,k)*snn(1,i)+kpt(2,k)*snn(2,i)+kpt(3,k)*snn(3,i)
           kr=kr*acell/16.d0
          CALL INIH2(HV,SNN(1,I),SNN(2,I),SNN(3,I))                    
!!************************************************************************
   er=(snn(1,i)*efield(1)+snn(2,i)*efield(2)+snn(3,i)*efield(3))
   
        DO  II=1,norb 
           DO  JJ=1,norb   
    A(ii+norb,jj+norb,k)= A(ii+norb,jj+norb,k)+HV(II,JJ)*DCMPLX(dcos(kr),dsin(kr))&
   &*er*acell/16.d0/uae
     
     A(ii,jj,k)= A(ii,jj,k)+HV(II,JJ)*DCMPLX(dcos(-kr),dsin(-kr)) &
   &*er*acell/16.d0/uae

          enddo
         enddo
        enddo
!******

!****** Third Neighboard
        do i=1,12
           kr=kpt(1,k)*tnn(1,i)+kpt(2,k)*tnn(2,i)+kpt(3,k)*tnn(3,i)
           kr=kr*acell/16.d0
          CALL INIH3(HV,tNN(1,i),tNN(2,i),tNN(3,i))                    
!!************************************************************************
     er =(tnn(1,i)*efield(1)+tnn(2,i)*efield(2)+tnn(3,i)*efield(3)) 

        DO  II=1,norb 
           DO  JJ=1,norb   
      A(ii,jj+norb,k)= A(ii,jj+norb,k)+HV(II,JJ)*DCMPLX(dcos(kr),dsin(kr)) &
     &*er*acell/16.d0/uae
     
      A(ii+norb,jj,k)= A(ii+norb,jj,k)-HV(II,JJ)*DCMPLX(dcos(-kr),dsin(-kr)) &
     &*er*acell/16.d0/uae
          enddo
         enddo
        enddo

!******
        enddo
  end subroutine evalA

  subroutine findneighboars(fnn,snn,tnn)
  implicit none
  
! Base vectors
  integer base1(3),base2(3)

! Lattice vectors
  integer l(3,3)

! Number of atoms to be allocated at once.

!****** Max range (in lattice vectors) spanned to create the nanowire
  integer , parameter :: maxL = 10

  integer atom1(3),atom2(3),d1,d2,i,k,j,d
  integer fnn(3,4),snn(3,12),tnn(3,12),ifn,isn,itn
  
  integer, parameter :: d1NN = 48
  integer, parameter :: d2NN = 128
  integer, parameter :: d3NN = 176
! base vectors in units of a/16
  base1=0

  base2(1)=4
  base2(2)=4
  base2(3)=4
  
! lattice vectors in units a/16

  l(1,1)=0
  l(2,1)=8
  l(3,1)=8
  
  l(1,2)=8
  l(2,2)=0
  l(3,2)=8
  
  l(1,3)=8
  l(2,3)=8
  l(3,3)=0
     
  ifn=1
  isn=1
  itn=1
  
  do i=-maxL,maxL
   do j=-maxL,maxL
    do k=-maxL,maxL
     do d=1,3
     atom1(d)=base1(d)+l(d,1)*i+l(d,2)*j+l(d,3)*k
     atom2(d)=base2(d)+l(d,1)*i+l(d,2)*j+l(d,3)*k
    enddo 

     d1=atom1(1)**2+atom1(2)**2+atom1(3)**2
     d2=atom2(1)**2+atom2(2)**2+atom2(3)**2
    
     if(d2==d1NN) then 
        do d=1,3
         fnn(d,ifn)=atom2(d)
        enddo
        ifn=ifn+1                    
     endif
    
     if(d1==d2NN) then 
      do d=1,3
        snn(d,isn)=atom1(d)
      enddo
       isn=isn+1                    
     endif

      if(d2==d3NN) then 
      do d=1,3
        tnn(d,itn)=atom2(d)
      enddo
       itn=itn+1                    
     endif
     
    enddo 
   enddo
  enddo
  end subroutine findneighboars

   !
! Returns abs(abs(i)-abs(j)).
!

    integer function difabs(i, j)

      implicit none

! Input/output variables.

      integer, intent(in) :: i, j

! Subroutine code.

      difabs = abs(abs(i)-abs(j))

    end function difabs

      SUBROUTINE INIH1(H,L,M,P)                                   
      use read_parms
      IMPLICIT none                                                  
      integer NF_AT
      complex(8) H(4,4)
      real(8) M_PI
      integer L,M,P
      real(8) sx,sy,sz,isign
      real(8) sinkx,sinky,sinkz,coskz,coskx,cosky
      complex(8) g0,g1,g2,g3

      parameter(M_PI=3.14159265358979323846)

! the off-diagonal parts


        sx=isign(L)
        sy=isign(M)
        sz=isign(P)
      
! SxS
        H(1,1) = ess111   !s1*s1
! S1 x P2
        H(1,2) = esx111*sx
        H(1,3) = esx111*sy
        H(1,4) = esx111*sz

! <Px|S>
        H(2,1) = -esx111*sx
! <Px|P>
        H(2,2) = exx111
        H(2,3) = exy111*sx*sy
        H(2,4) = exy111*sx*sz

! <Py|S>
        H(3,1) = -esx111*sy
! <Py|P>
        H(3,2) = exy111*sy*sx
        H(3,3) = exx111
        H(3,4) = exy111*sy*sz

! <Pz|S>
        H(4,1) = -esx111*sz
! <Pz|P>
        H(4,2) = exy111*sz*sx
        H(4,3) = exy111*sz*sy
        H(4,4) = exx111

      RETURN                                          
      END  


      SUBROUTINE INIH2(H,L,M,P)                                   
      use read_parms
      IMPLICIT none                                                  
      complex(8) H(4,4)
      integer, intent(in) :: L,M,P
      real(8) dx,dy,dz,isign


! the off-diagonal parts

        dx=isign(L)
        dy=isign(M)
        dz=isign(P)

        H(1,1) =  ess220
        H(1,2) =  esx220*dx
        H(1,3) =  esx220*dy
        H(1,4) =  esx220*dz

        H(2,1) = -esx220*dx
        if (L /= 0) then
          H(2,2) =  exx220
        else
          H(2,2) =  exx022
        endif
        H(2,3) =  exy220*dx*dy
        H(2,4) =  exy220*dx*dz

        H(3,1) = -esx220*dy
        H(3,2) =  exy220*dy*dx
        if (M /= 0) then
          H(3,3) =  exx220
        else
          H(3,3) =  exx022
        endif
        H(3,4) =  exy220*dy*dz

        H(4,1) = -esx220*dz
        H(4,2) =  exy220*dz*dx
        H(4,3) =  exy220*dz*dy
        if (P /= 0) then
          H(4,4) =  exx220
        else
          H(4,4) =  exx022
        endif
      RETURN                                          
      END 

      SUBROUTINE INIH3(H,L,M,P)                                   
      use read_parms
      IMPLICIT none                                                  
      integer NF_AT,ix,iy,iz,difabs
      complex(8) H(4,4)
      real(8) M_PI
      integer, intent(in) :: L,M,P
      real(8) dx,dy,dz,isign
      real(8) sinkx,sinky,sinkz,coskz,coskx,cosky
      complex(8) g0,g1,g2,g3

      parameter(M_PI=3.14159265358979323846)

! the off-diagonal parts

        ix=L
        iy=M
        iz=P
      
        dx=isign(L)
        dy=isign(M)
        dz=isign(P)

        H(1,1) =  ess311
          if (difabs(iy, iz) == 0) then
            H(1,2) =  esx311*dx
          else
            H(1,2) =  esx113*dx
          endif
          if (difabs(ix, iz) == 0) then
            H(1,3) =  esx311*dy
          else
            H(1,3) =  esx113*dy
          endif
          if (difabs(ix, iy) == 0) then
            H(1,4) =  esx311*dz
          else
            H(1,4) =  esx113*dz
          endif

          if (difabs(iy, iz) == 0) then
            H(2,1) = -esx311*dx
            H(2,2) =  exx311
          else
            H(2,1) = -esx113*dx
            H(2,2) =  exx113
          endif
          if (difabs(ix, iy) == 0) then
            H(2,3) =  exy113*dx*dy
          else
            H(2,3) =  exy311*dx*dy
          endif
          if (difabs(ix, iz) == 0) then
            H(2,4) =  exy113*dx*dz
          else
            H(2,4) =  exy311*dx*dz
          endif

          if (difabs(ix, iz) == 0) then
            H(3,1) = -esx311*dy
            H(3,3) =  exx311
          else
            H(3,1) = -esx113*dy
            H(3,3) =  exx113
          endif
          if (difabs(ix, iy) == 0) then
            H(3,2) =  exy113*dy*dx
          else
            H(3,2) =  exy311*dy*dx
          endif
          if (difabs(iy, iz) == 0) then
            H(3,4) =  exy113*dy*dz
          else
            H(3,4) =  exy311*dy*dz
          endif

          if (difabs(ix, iy) == 0) then
            H(4,1) = -esx311*dz
            H(4,4) =  exx311
          else
            H(4,1) = -esx113*dz
            H(4,4) =  exx113
          endif
          if (difabs(iz, ix) == 0) then
            H(4,2) =  exy113*dx*dz
          else
            H(4,2) =  exy311*dx*dz
          endif
          if (difabs(iz, iy) == 0) then
            H(4,3) =  exy113*dy*dz
          else
            H(4,3) =  exy311*dy*dz
          endif
        
      RETURN                                          
      END 

       real(8) function isign(i)

      implicit none

! Input/output variables.

      integer, intent(in) :: i

! Subroutine code.

      if (i > 0) then
        isign = +1.d0
      else if (i < 0) then
        isign = -1.d0
      else
        isign =  0.d0
      endif

      end function isign 
