module my_types
  save
   integer,parameter ::  dp = kind(1.0d0), n_rap = 8
   real(dp),parameter :: pi=3.14159265358979_dp, pid2 = pi / 2.0_dp, a_Si = 5.4307_dp
   real(dp),parameter :: a_B = 0.529177_dp, E_B = 13.6058_dp
   real(dp),parameter :: eps = 1.0e-7,pid4 = pi/4.0_dp

   type vect
    real(dp) :: r(3)
   end type

   type atom
    type(vect)        :: pos
    character(len =2) :: name
   end type

   type atomic_orbital
    type(atom) :: at
    integer :: v,phase
   end type

   type hamilt_el
    type(atom) :: at
    integer :: v1,v2
    real(dp) :: value
   end type

   type psi
    real(dp) :: val(n_rap)
    integer :: row(n_rap)
   end type   

   interface assignment( = )
     module procedure vect_assign,atom_assign,orb_assign,psi_assign
   end interface

   interface operator( == )
     module procedure at_equal,orb_equal,vect_equal,psi_equal
   end interface

   interface operator( + )
     module procedure vect_sum
   end interface

   interface operator( * )
     module procedure const_per_vect
   end interface

   contains
     subroutine vect_assign(v_2,v_1)
       type(vect), intent(in) :: v_1
       type(vect), intent(out) :: v_2

       v_2%r = v_1%r
     return
     end subroutine

     subroutine psi_assign(psi_2,psi_1)
       type(psi), intent(in) :: psi_1
       type(psi), intent(out) :: psi_2

       psi_2%val = psi_1%val
       psi_2%row = psi_1%row

     return
     end subroutine


     subroutine atom_assign(at_2,at_1)
       type(atom), intent(in) :: at_1
       type(atom), intent(out) :: at_2

       call vect_assign(at_2%pos,at_1%pos)
       at_2%name = at_1%name
     return
     end subroutine


     subroutine orb_assign(orb_2,orb_1)
       type(atomic_orbital), intent(in) :: orb_1
       type(atomic_orbital), intent(out) :: orb_2

       call atom_assign(orb_2%at,orb_1%at)
       orb_2%v = orb_1%v
       orb_2%phase = orb_1%phase
     return
     end subroutine

     function vect_equal(v_1,v_2) result (eq)
       type(vect), intent(in) :: v_1,v_2
       logical :: eq

       eq = (v_1%r(1) == v_2%r(1) .and.  &
       v_1%r(2) == v_2%r(2) .and. v_1%r(3) == v_2%r(3))
     return
     end function

     function at_equal(at_1,at_2) result (eq)
       type(atom), intent(in) :: at_1,at_2
       logical :: eq
       eq = vect_equal(at_1%pos,at_2%pos)
       eq = eq .and. (at_1%name == at_2%name)
     return
     end function

     function orb_equal(orb_1,orb_2) result (eq)
       type(atomic_orbital), intent(in) :: orb_1,orb_2
       logical :: eq
       eq = at_equal(orb_1%at,orb_2%at)

       eq = eq .and. (orb_1%v == orb_2%v)

     return
     end function

     function col_equal(col_1,col_2) result (eq)
      real(dp),intent(in) :: col_1(:),col_2(:)
      logical :: eq
      integer ::s

      s = size(col_1)
      eq = .true.

      if(size(col_2) /= s) then
       eq = .false.
       return
      endif

      i = 0
      do while(eq .and. i<s)
       i = i + 1
       eq = eq .and. (col_1(i) == col_2(i))
      enddo
     return
     end function

     function psi_equal(psi_1,psi_2) result (eq)
     type(psi),intent(in) :: psi_1,psi_2
     logical :: eq
 
     eq = .true.
     i = 0
     do while(eq .and. i< n_rap)
      i = i + 1
      eq = eq .and. (psi_1%val(i) == psi_2%val(i))
      eq = eq .and. (psi_1%row(i) == psi_2%row(i))
     enddo
     return
     end function    

     function vect_sum(v_1,v_2) result (v_out)
       type(vect), intent(in) :: v_1,v_2
       type(vect) :: v_out
       v_out%r = v_1%r + v_2%r
     return
     end function

     function const_per_vect(v_in,alpha) result (v_out)
       type(vect), intent(in) :: v_in
       integer, intent(in) :: alpha
       type(vect) :: v_out

       v_out%r = alpha * v_in%r
     return
     end function

end module
