
      module read_parms
      implicit none
      integer, parameter :: dp=8

!!******* Parameters of Phys. Rev. B Vol 62 Number 8 page 5109      
      real(dp) ESS000
      real(dp) EPP000
!!*******************
      real(dp) ESS111
      real(dp) ESX111
      real(dp) EXX111
      real(dp) EXY111
!!*******************
      real(dp) ESS220
      real(dp) ESX220
      real(dp) ESX022
      real(dp) EXX220
      real(dp) EXX022
      real(dp) EXY220
      real(dp) EXY022
      real(dp) ESS311
      real(dp) ESX311
      real(dp) ESX113
      real(dp) EXX311
      real(dp) EXX113
      real(dp) EXY311
      real(dp) EXY113 
      end module read_parms


      program main
      implicit none
      real(8) p(13),func
      p(1) =   -6.17334 
      p(2) =    2.39585
!!*************************************************
      p(3) =   -1.78516
      p(4) =    0.78088 
      p(5) =    0.35657 
      p(6) =    1.47649
!!*************************************************
      p(7) =    0.23010
      p(8) =   -0.21608 
      p(9) =   -0.02496
      p(10) =    0.02286 
      p(11) =   -0.24379
      p(12) =   -0.05462 
      p(13) =   -0.12754
      call objfun(p,func)
      end program main
      
      subroutine objfun(p,func)
      use read_parms
      implicit none
      real(dp) p(13),func
      integer lwork
      integer n,i,info,l,ndim,c,ix,iy,iz
      integer j,k,a,b,ncell,d
      complex(8) , ALLOCATABLE :: work(:),H(:,:)
      real(8) , ALLOCATABLE :: rwork(:),eigv(:,:)
      
      real(8) step,kvec(3),M_PI,x,s2,s3,s6,xn,xl

      real(8) a_fcc(3,3),b_fcc(3,3),acell
      real(8), parameter :: ual = 0.5291772083d0 
      parameter(M_PI=3.14159265358979323846)
     
       ESS000=p(1)
       EPP000=p(2)
!!*******************
       ESS111=p(3)
       ESX111=p(4)
       EXX111=p(5)
       EXY111=p(6)
!!*******************
       ESS220=p(7)
       ESX220=p(8)
       ESX022=p(9)
       EXX220=p(10)
       EXX022=p(11)
       EXY220=p(12)
       EXY022=p(13)
!!*******************
        
        c=1
             ndim=8
      lwork=ndim*100
      
      
      write(*,*) ' Number of points for Band Structure'
      read(5,*) n
      step = 2.d0*M_PI/dble(n) 
!* Longueur du trace Gamma-X-W-L-Gamma-K-X                               
      
      allocate(work(lwork),rwork(lwork),H(ndim,ndim),eigv(ndim,9*n))

      XN = FLOAT(N)                                                     
      S2=DSQRT(2.D+00)
      S3=DSQRT(3.D+00)               
      S6=DSQRT(6.D+00)               
      XL = XN*(0.5*S3+1.d0+0.5+0.25*S2+0.25*S6+0.5*S2+0.5+S2) 
      
      open(unit=10,file='bandstructure.dat',form='formatted'&
     &,status='unknown')

      X = -S3/XL 
     
      WRITE (6,*)'Axe Gamma-X'
      WRITE (10,*)'# G'   
      do i=0,N,1
      iy=0
      iz=0
      ix=i
      x=x+1.d0/XL
      kvec(1)=step*ix
      kvec(2)=step*iy
      kvec(3)=step*iz
      H=DCMPLX(0.d0,0.d0)
      call fill_diagonal(H)
      call fill_offdiagonal(kvec,H)
      call zheev ('N','U',ndim,H,ndim,eigv(1,c),work,lwork,rwork, info )
      c=c+1
      enddo 

      
      WRITE (6,*)' Axe X-W'
      WRITE (10,*)'# X'   
      do i=1,N/2
      iy=i
      iz=0
      ix=N
      x=x+1.d0/XL
      kvec(1)=step*ix
      kvec(2)=step*iy
      kvec(3)=step*iz
      H=DCMPLX(0.d0,0.d0)
      call fill_diagonal(H)
      call fill_offdiagonal(kvec,H)
      call zheev ('N','U',ndim,H,ndim,eigv(1,c),work,lwork,rwork, info )
      c=c+1
      enddo 

      WRITE (6,*)' Axe W-L'
      WRITE (10,*)'# W'   
      do i=1,N/2
      ix=N-i
      iy=N/2
      iz=i
      x=x+S2/XL
      kvec(1)=step*ix
      kvec(2)=step*iy
      kvec(3)=step*iz
      H=DCMPLX(0.d0,0.d0)
      call fill_diagonal(H)
      call fill_offdiagonal(kvec,H)
      call zheev ('N','U',ndim,H,ndim,eigv(1,c),work,lwork,rwork, info )
      c=c+1
      enddo  

      
      WRITE (6,*)'L-Gamma'
      WRITE (10,*)'# L'   
      do i=n/2,0,-1
      iy=i
      ix=i
      iz=i
      x=x+S3/xl
      kvec(1)=step*ix
      kvec(2)=step*iy
      kvec(3)=step*iz
      H=DCMPLX(0.d0,0.d0)
      call fill_diagonal(H)
      call fill_offdiagonal(kvec,H)
      call zheev ('N','U',ndim,H,ndim,eigv(1,c),work,lwork,rwork, info )
      c=c+1
      enddo 
     
      WRITE (6,*)' Axe Gamma-K'
      WRITE (10,*)'# X'   
      do i=1,N
      ix=4*i
      iy=i
      iz=i
      x=x+S2/XL
      kvec(1)=step*ix/4.d0
      kvec(2)=step*iy/4.d0
      kvec(3)=step*iz/4.d0
      H=DCMPLX(0.d0,0.d0)
      call fill_diagonal(H)
      call fill_offdiagonal(kvec,H)
      call zheev ('N','U',ndim,H,ndim,eigv(1,c),work,lwork,rwork, info )
      c=c+1
      enddo 

      
      do i=1,7/2*n+1
      write(10,'(i4,100e15.7)') i,(eigv(l,i),l=1,ndim)
      enddo
      close(10)

      end subroutine objfun

      subroutine fill_diagonal(H)  
      use read_parms
      implicit none
      complex(8) H(8,8)
      integer IL,I,J
        H(1,1) = DCMPLX(ESS000,0.d0)
        H(2,2) = DCMPLX(EPP000,0.d0)
        H(3,3) = DCMPLX(EPP000,0.d0)
        H(4,4) = DCMPLX(EPP000,0.d0)
        H(5,5) = DCMPLX(ESS000,0.d0)
        H(6,6) = DCMPLX(EPP000,0.d0)
        H(7,7) = DCMPLX(EPP000,0.d0)
        H(8,8) = DCMPLX(EPP000,0.d0)
      end subroutine fill_diagonal

      subroutine fill_offdiagonal(kvec,H)  
      use read_parms
      implicit none
      integer, parameter :: norb=4
      complex(8) H(norb*2,norb*2),HV(norb,norb)
      real(8) kvec(3),X,Y,Z,XN,PHI
      integer L,M,P,d
      integer IL,I,J,IV,II,JJ,FNN(3,4),SNN(3,12),TNN(3,12)
      complex(8) temp
 
      call findneighboars(fnn,snn,tnn)
     
       temp=DCMPLX(0.d0,0.d0)
      
      
      DO IV=1,4                                           
! Neighboards in A units                                                
        X = FNN(1,IV)/16.D+00                                           
        Y = FNN(2,IV)/16.D+00                                           
        Z = FNN(3,IV)/16.D+00                                           
        XN = DSQRT( X**2+Y**2+Z**2)       

      PHI = kvec(1)*X+kvec(2)*Y+kvec(3)*Z    
      
       CALL INIH1(HV,FNN(1,IV),FNN(2,IV),FNN(3,IV))                    
!************************************************************************
        DO  II=1,norb                                                
          DO  JJ=1,norb                                              
   H(II,norb+JJ) = H(II,norb+JJ)+HV(II,JJ)*DCMPLX(dcos(phi),dsin(phi))
          enddo
         enddo
!************************************************************************
         enddo

        DO IV=1,12                                           
! Neighboards in A units                                                
         X = SNN(1,IV)/16.D+00                                          
         Y = SNN(2,IV)/16.D+00                                          
         Z = SNN(3,IV)/16.D+00                                           
        XN = DSQRT( X**2+Y**2+Z**2)       
       PHI = kvec(1)*X+kvec(2)*Y+kvec(3)*Z    

       CALL INIH2(HV,SNN(1,IV),SNN(2,IV),SNN(3,IV))                    
!!************************************************************************
        DO  II=1,norb 
           DO  JJ=1,norb   
    H(II,JJ) = H(II,JJ)+HV(II,JJ)*DCMPLX(dcos(phi),dsin(phi))
    H(II+norb,JJ+norb) = H(II+norb,JJ+norb)+HV(II,JJ)*DCMPLX(dcos(phi),dsin(phi))
          enddo
         enddo
!!************************************************************************
         enddo
         return
      end subroutine fill_offdiagonal
              

      SUBROUTINE INIH1(H,L,M,P)                                   
      use read_parms
      IMPLICIT none                                                  
      integer NF_AT
      complex(8) H(4,4)
      real(8) M_PI
      integer L,M,P
      real(8) sx,sy,sz,isign
      real(8) sinkx,sinky,sinkz,coskz,coskx,cosky
      complex(8) g0,g1,g2,g3

      parameter(M_PI=3.14159265358979323846)

! the off-diagonal parts


        sx=isign(L)
        sy=isign(M)
        sz=isign(P)
      
! SxS
        H(1,1) = ess111   !s1*s1
! S1 x P2
        H(1,2) = esx111*sx
        H(1,3) = esx111*sy
        H(1,4) = esx111*sz

! <Px|S>
        H(2,1) = -esx111*sx
! <Px|P>
        H(2,2) = exx111
        H(2,3) = exy111*sx*sy
        H(2,4) = exy111*sx*sz

! <Py|S>
        H(3,1) = -esx111*sy
! <Py|P>
        H(3,2) = exy111*sy*sx
        H(3,3) = exx111
        H(3,4) = exy111*sy*sz

! <Pz|S>
        H(4,1) = -esx111*sz
! <Pz|P>
        H(4,2) = exy111*sz*sx
        H(4,3) = exy111*sz*sy
        H(4,4) = exx111

      RETURN                                          
      END  

      SUBROUTINE INIH2(H,L,M,P)                                   
      use read_parms
      IMPLICIT none                                                  
      complex(8) H(4,4)
      integer, intent(in) :: L,M,P
      real(8) dx,dy,dz,isign


! the off-diagonal parts

        dx=isign(L)
        dy=isign(M)
        dz=isign(P)

        H(1,1) =  ess220
        H(1,2) =  esx220*dx
        H(1,3) =  esx220*dy
        H(1,4) =  esx220*dz

        H(2,1) = -esx220*dx
        if (L /= 0) then
          H(2,2) =  exx220
        else
          H(2,2) =  exx022
        endif
        H(2,3) =  exy220*dx*dy
        H(2,4) =  exy220*dx*dz

        H(3,1) = -esx220*dy
        H(3,2) =  exy220*dy*dx
        if (M /= 0) then
          H(3,3) =  exx220
        else
          H(3,3) =  exx022
        endif
        H(3,4) =  exy220*dy*dz

        H(4,1) = -esx220*dz
        H(4,2) =  exy220*dz*dx
        H(4,3) =  exy220*dz*dy
        if (P /= 0) then
          H(4,4) =  exx220
        else
          H(4,4) =  exx022
        endif
      RETURN                                          
      END 

      SUBROUTINE INIH3(H,L,M,P)                                   
      use read_parms
      IMPLICIT none                                                  
      integer NF_AT,ix,iy,iz,difabs
      complex(8) H(4,4)
      real(8) M_PI
      integer, intent(in) :: L,M,P
      real(8) dx,dy,dz,isign
      real(8) sinkx,sinky,sinkz,coskz,coskx,cosky
      complex(8) g0,g1,g2,g3

      parameter(M_PI=3.14159265358979323846)

! the off-diagonal parts

        ix=L
        iy=M
        iz=P
      
        dx=isign(L)
        dy=isign(M)
        dz=isign(P)

        H(1,1) =  ess311
          if (difabs(iy, iz) == 0) then
            H(1,2) =  esx311*dx
          else
            H(1,2) =  esx113*dx
          endif
          if (difabs(ix, iz) == 0) then
            H(1,3) =  esx311*dy
          else
            H(1,3) =  esx113*dy
          endif
          if (difabs(ix, iy) == 0) then
            H(1,4) =  esx311*dz
          else
            H(1,4) =  esx113*dz
          endif

          if (difabs(iy, iz) == 0) then
            H(2,1) = -esx311*dx
            H(2,2) =  exx311
          else
            H(2,1) = -esx113*dx
            H(2,2) =  exx113
          endif
          if (difabs(ix, iy) == 0) then
            H(2,3) =  exy113*dx*dy
          else
            H(2,3) =  exy311*dx*dy
          endif
          if (difabs(ix, iz) == 0) then
            H(2,4) =  exy113*dx*dz
          else
            H(2,4) =  exy311*dx*dz
          endif

          if (difabs(ix, iz) == 0) then
            H(3,1) = -esx311*dy
            H(3,3) =  exx311
          else
            H(3,1) = -esx113*dy
            H(3,3) =  exx113
          endif
          if (difabs(ix, iy) == 0) then
            H(3,2) =  exy113*dy*dx
          else
            H(3,2) =  exy311*dy*dx
          endif
          if (difabs(iy, iz) == 0) then
            H(3,4) =  exy113*dy*dz
          else
            H(3,4) =  exy311*dy*dz
          endif

          if (difabs(ix, iy) == 0) then
            H(4,1) = -esx311*dz
            H(4,4) =  exx311
          else
            H(4,1) = -esx113*dz
            H(4,4) =  exx113
          endif
          if (difabs(iz, ix) == 0) then
            H(4,2) =  exy113*dx*dz
          else
            H(4,2) =  exy311*dx*dz
          endif
          if (difabs(iz, iy) == 0) then
            H(4,3) =  exy113*dy*dz
          else
            H(4,3) =  exy311*dy*dz
          endif
        
      RETURN                                          
      END 
          
     real(8) function isign(i)

      implicit none

! Input/output variables.

      integer, intent(in) :: i

! Subroutine code.

      if (i > 0) then
        isign = +1.d0
      else if (i < 0) then
        isign = -1.d0
      else
        isign =  0.d0
      endif

      end function isign   


  subroutine findneighboars(fnn,snn,tnn)
  implicit none
  
! Base vectors
  integer base1(3),base2(3)

! Lattice vectors
  integer l(3,3)

! Number of atoms to be allocated at once.

!****** Max range (in lattice vectors) spanned to create the nanowire
  integer , parameter :: maxL = 10

  integer atom1(3),atom2(3),d1,d2,i,k,j,d
  integer fnn(3,4),snn(3,12),tnn(3,12),ifn,isn,itn
  
  integer, parameter :: d1NN = 48
  integer, parameter :: d2NN = 128
  integer, parameter :: d3NN = 176
! base vectors in units of a/16
  base1=0

  base2(1)=4
  base2(2)=4
  base2(3)=4
  
! lattice vectors in units a/16

  l(1,1)=0
  l(2,1)=8
  l(3,1)=8
  
  l(1,2)=8
  l(2,2)=0
  l(3,2)=8
  
  l(1,3)=8
  l(2,3)=8
  l(3,3)=0
     
  ifn=1
  isn=1
  itn=1
  
  do i=-maxL,maxL
   do j=-maxL,maxL
    do k=-maxL,maxL
     do d=1,3
     atom1(d)=base1(d)+l(d,1)*i+l(d,2)*j+l(d,3)*k
     atom2(d)=base2(d)+l(d,1)*i+l(d,2)*j+l(d,3)*k
    enddo 

     d1=atom1(1)**2+atom1(2)**2+atom1(3)**2
     d2=atom2(1)**2+atom2(2)**2+atom2(3)**2
    
     if(d2==d1NN) then 
        do d=1,3
         fnn(d,ifn)=atom2(d)
        enddo
        ifn=ifn+1                    
     endif
    
     if(d1==d2NN) then 
      do d=1,3
        snn(d,isn)=atom1(d)
      enddo
       isn=isn+1                    
     endif

      if(d2==d3NN) then 
      do d=1,3
        tnn(d,itn)=atom2(d)
      enddo
       itn=itn+1                    
     endif
     
    enddo 
   enddo
  enddo
  end subroutine findneighboars

   !
! Returns abs(abs(i)-abs(j)).
!

    integer function difabs(i, j)

      implicit none

! Input/output variables.

      integer, intent(in) :: i, j

! Subroutine code.

      difabs = abs(abs(i)-abs(j))

    end function difabs

